@@ -5,6 +5,7 @@ import types
 import copy
 
 from django import forms
+from django.forms.forms import NON_FIELD_ERRORS
 from django.core.validators import EMPTY_VALUES
 from django.db import models
 from django.db.models.fields import FieldDoesNotExist
@@ -48,6 +49,16 @@ if version_info < (2, 7, 0):
     copy._deepcopy_dispatch[types.MethodType] = _deepcopy_method
 
 
+class STRICTNESS(object):
+    """
+    Values of False & True chosen for backward compatability reasons.
+    Originally, these were the only options.
+    """
+    IGNORE = False
+    RETURN_NO_RESULTS = True
+    RAISE_VALIDATION_ERROR = "RAISE"
+
+
 def get_declared_filters(bases, attrs, with_base_filters=True):
     filters = []
     for filter_name, obj in list(attrs.items()):
@@ -79,8 +90,12 @@ def get_model_field(model, f):
         except FieldDoesNotExist:
             return None
         if isinstance(rel, ForeignObjectRel):
-            model = rel.model
-            opts = rel.opts
+            if hasattr(rel, "related_model"):
+                # django >= 1.8 (ForeignObjectRel)
+                opts = rel.related_model._meta
+            else:
+                # django < 1.8 (RelatedObject)
+                opts = rel.opts
         else:
             model = rel.rel.to
             opts = model._meta
@@ -132,6 +147,31 @@ def filters_for_model(model, fields=None, exclude=None, filter_for_field=None,
     return field_dict
 
 
+def get_full_clean_override(together):
+    def full_clean(form):
+        
+        def add_error(message):
+            try:
+                form.add_error(None, message)
+            except AttributeError:
+                form._errors[NON_FIELD_ERRORS] = message
+        
+        def all_valid(fieldset):
+            cleaned_data = form.cleaned_data
+            count = len([i for i in fieldset if cleaned_data.get(i)])
+            return 0 < count < len(fieldset)
+        
+        super(form.__class__, form).full_clean()
+        message = 'Following fields must be together: %s'
+        if isinstance(together[0], (list, tuple)):
+            for each in together:
+                if all_valid(each):
+                    return add_error(message % ','.join(each))
+        elif all_valid(together):
+            return add_error(message % ','.join(together))
+    return full_clean
+
+
 class FilterSetOptions(object):
     def __init__(self, options=None):
         self.model = getattr(options, 'model', None)
@@ -141,6 +181,8 @@ class FilterSetOptions(object):
         self.order_by = getattr(options, 'order_by', False)
 
         self.form = getattr(options, 'form', forms.Form)
+        
+        self.together = getattr(options, 'together', None)
 
 
 class FilterSetMetaclass(type):
@@ -266,7 +308,8 @@ FILTER_FOR_DBFIELD_DEFAULTS = {
 class BaseFilterSet(object):
     filter_overrides = {}
     order_by_field = ORDER_BY_FIELD
-    strict = True
+    # What to do on on validation errors
+    strict = STRICTNESS.RETURN_NO_RESULTS
 
     def __init__(self, data=None, queryset=None, prefix=None, strict=None):
         self.is_bound = data is not None
@@ -302,9 +345,13 @@ class BaseFilterSet(object):
         if not hasattr(self, '_qs'):
             valid = self.is_bound and self.form.is_valid()
 
-            if self.strict and self.is_bound and not valid:
-                self._qs = self.queryset.none()
-                return self._qs
+            if self.is_bound and not valid:
+                if self.strict == STRICTNESS.RAISE_VALIDATION_ERROR:
+                    raise forms.ValidationError(self.form.errors)
+                elif bool(self.strict) == STRICTNESS.RETURN_NO_RESULTS:
+                    self._qs = self.queryset.none()
+                    return self._qs
+                # else STRICTNESS.IGNORE...  ignoring
 
             # start with all the results and filter from there
             qs = self.queryset.all()
@@ -317,13 +364,12 @@ class BaseFilterSet(object):
                     try:
                         value = self.form.fields[name].clean(raw_value)
                     except forms.ValidationError:
-                        # for invalid values either:
-                        # strictly "apply" filter yielding no results and get outta here
-                        if self.strict:
+                        if self.strict == STRICTNESS.RAISE_VALIDATION_ERROR:
+                            raise
+                        elif bool(self.strict) == STRICTNESS.RETURN_NO_RESULTS:
                             self._qs = self.queryset.none()
                             return self._qs
-                        else:  # or ignore this filter altogether
-                            pass
+                        # else STRICTNESS.IGNORE...  ignoring
 
                 if value is not None:  # valid & clean data
                     qs = filter_.filter(qs, value)
@@ -359,6 +405,8 @@ class BaseFilterSet(object):
             fields[self.order_by_field] = self.ordering_field
             Form = type(str('%sForm' % self.__class__.__name__),
                         (self._meta.form,), fields)
+            if self._meta.together:
+                Form.full_clean = get_full_clean_override(self._meta.together)
             if self.is_bound:
                 self._form = Form(self.data, prefix=self.form_prefix)
             else:
